import 'package:example/dto.dart';
import 'package:flutter/foundation.dart';
import 'package:flutterx_http/flutterx_http.dart';
import 'package:flutterx_utils/flutterx_utils.dart';
import 'package:http/http.dart';

class OpenWeather {
  final String _apiKey;
  final XHttpClient _client;

  OpenWeather(this._apiKey)
      : _client = XHttpClient(baseUrl: 'https://cors-anywhere.herokuapp.com/https://api.openweathermap.org/data/2.5');

  Future<WeatherInfo> getWeather({required String city, Units units = Units.standard, CancellationToken? cancel}) =>
      _client
          .get('/weather', params: {'q': city, 'units': describeEnum(units)}, onRequest: _addApiKey, cancel: cancel)
          .asJsonObject(WeatherInfo.fromJson);

  String imageUrl(WWeather weather) => 'https://openweathermap.org/img/wn/${weather.icon}@2x.png';

  BaseRequest _addApiKey(BaseRequest request) =>
      request.copy(url: request.url.appendQueryParameters({'appid': _apiKey}));
}

enum Units { standard, metric, imperial }

extension UnitsExt on Units {
  String get unit {
    switch (this) {
      case Units.standard:
        return '°K';
      case Units.metric:
        return '°C';
      case Units.imperial:
        return '°F';
    }
  }
}
