import 'dart:async';
import 'dart:convert';
import 'dart:typed_data';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutterx_http/src/x_http_client.dart';
import 'package:flutterx_utils/flutterx_utils.dart';
import 'package:http/http.dart';

extension ResponseExt on Future<Response> {
  Future<Uint8List> asBytes() => then((response) => response.bodyBytes);

  Future<EncodedData> asEncodedData() => then((response) => EncodedData(response.bodyBytes, response.encoding));

  Future<String> asText() => then((response) => response.body);

  Future<T> asJson<T>(FromJson<T> fromJson) => then((response) => fromJson(jsonDecode(response.body)));

  Future<T> asJsonObject<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJson(json));

  Future<List<T>> asJsonList<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJsonList(json, fromJson));

  Future<Map<String, T>> asJsonMap<T>(FromJsonObject<T> fromJson) => asJson((json) => fromJsonMap(json, fromJson));
}

@immutable
class EncodedData {
  final Uint8List data;
  final Encoding encoding;

  const EncodedData(this.data, [this.encoding = utf8]);

  String decode() => encoding.decode(data);
}
